@@ -4,23 +4,32 @@ LogitCD <- function(X, Y, penalty=c("network", "mcp", "lasso"), lamb.1=NULL, lam
 {
   n = nrow(X); p = ncol(X);
   x = as.matrix(X); y = as.matrix(Y)
+  vname = colnames(x)
   b0 = rep(0, p+1)
   method = substr(penalty, 1, 1)
   #---------------------------------------------- Main Loop -----------------------------------------
-  if(standardize) x = scale(x, scale = apply(x, 2, function(t) stats::sd(t)*sqrt((n-1)/n)))
-  if(penalty == "network") a = Adjacency(x) else a = as.matrix(0)
+  V0 = apply(x, 2, function(t) stats::sd(t)*sqrt((n-1)/n));
+  if(any(V0==0) & (penalty == "network")) stop("X columns have standard deviation equal zero");
+  if(standardize){
+    V0[V0==0|is.na(V0)]=1
+    x = scale(x, center = TRUE, scale = V0)
+  }
+
+  a = Adjacency(x)
   x = cbind(rep(1,n), x)
-  init = match.arg(init, choices = c("zero","elnet"))
+  init = match.arg(init, choices = c("elnet","zero"))
   if(init == "elnet") b0 = initiation(x, y, alpha.i, "binomial")
 
-  b = RunLogit(x, y, lamb.1, lamb.2, b0, r, a, p, alpha, method)
+  triRowAbsSums = rowSums(abs(a*upper.tri(a, diag = FALSE)))
+  b = RunLogit(x, y, lamb.1, lamb.2, b0, r, a, triRowAbsSums, p, alpha, method)
   b = as.numeric(b)
-  vname = colnames(x)
+
   if(!is.null(vname)){
     names(b) = c("Intercept", vname)
   }else{
     names(b) = c("Intercept", paste("v", seq = (1:p), sep=""))
   }
 
-  return(drop(b))
+  sub = which(b[-1]!=0)
+  out = list(b=drop(b), Adj=a[sub,sub,drop=FALSE])
 }
